// Copyright 2019 the V8 project authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

assertDoesNotThrow("try { } catch (e) { var e; for (var e of []) {} }")
