// Copyright 2025 the V8 project authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

WScript.LoadScriptFile("WasmSpec/testsuite/harness/wasm-constants.js");
WScript.LoadScriptFile("WasmSpec/testsuite/harness/wasm-module-builder.js");
