// Copyright 2020 the V8 project authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

__f_1();
function __f_1() {
  for (var __v_3 = 0; __v_3 < 100000; __v_3++) {
  }
}
__f_1();
