// Copyright 2024 the V8 project authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

// Flags: --fuzzing --allow-natives-syntax --wasm-staging

const generated_module = %WasmGenerateRandomModule();
if (typeof WebAssembly == "object") {
  assertInstanceof(generated_module, WebAssembly.Module);
} else {
  // Jitless etc.
  assertSame(generated_module, undefined);
}
